'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _urlSearchParams = require('url-search-params');

var _urlSearchParams2 = _interopRequireDefault(_urlSearchParams);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var mimeType = {
  form: 'application/x-www-form-urlencoded',
  json: 'application/json'
};

function createResponse(axiosResponse) {
  if ((typeof axiosResponse === 'undefined' ? 'undefined' : _typeof(axiosResponse)) !== 'object') {
    return axiosResponse;
  }

  return {
    status: axiosResponse.status,
    statusText: axiosResponse.statusText,
    headers: axiosResponse.headers,
    data: axiosResponse.data
  };
}

function createError(axiosError) {
  return {
    name: axiosError.name,
    code: axiosError.code,
    message: axiosError.message,
    response: createResponse(axiosError.response)
  };
}

_axios2.default.interceptors.response.use(function (response) {
  return createResponse(response);
}, function (error) {
  return Promise.reject(createError(error));
});

var methods = {
  create: 'POST',
  read: 'GET',
  update: 'PUT',
  delete: 'DELETE'
};

function getUrl(requestConfig) {
  return requestConfig.resource;
}

function getMethod(requestConfig) {
  return methods[requestConfig.operation];
}

function getBaseUrl(requestConfig) {
  return requestConfig.url;
}

function getHeaders(requestConfig) {
  var headers = {};

  if (_config2.default.hasOption(requestConfig, 'type')) {
    var type = requestConfig.type;

    if (type === mimeType.form || type === mimeType.json) {
      type = type + ';charset=utf-8';
    }

    headers['Content-Type'] = type;
  }

  if (_config2.default.hasOption(requestConfig, 'token')) {
    headers.Authorization = 'Bearer ' + requestConfig.token;
  }

  return headers;
}

function getParams(requestConfig) {
  return requestConfig.params;
}

function getFormData(data) {
  var params = new _urlSearchParams2.default();

  Object.keys(data).forEach(function (key) {
    return params.set(key, data[key]);
  });

  return params.toString();
}

function getData(requestConfig) {
  if (requestConfig.type === mimeType.form) {
    return getFormData(requestConfig.data);
  }

  return requestConfig.data;
}

function getAuth(requestConfig) {
  if (_config2.default.hasOption(requestConfig, 'token')) {
    return null;
  }

  return {
    username: requestConfig.username,
    password: requestConfig.password
  };
}

function getConfig(requestConfig) {
  return {
    url: getUrl(requestConfig),
    method: getMethod(requestConfig),
    baseURL: getBaseUrl(requestConfig),
    headers: getHeaders(requestConfig),
    params: getParams(requestConfig),
    data: getData(requestConfig),
    auth: getAuth(requestConfig)
  };
}

function sendRequest(requestConfig) {
  return (0, _axios2.default)(getConfig(requestConfig));
}

exports.default = {
  getUrl: getUrl,
  getMethod: getMethod,
  getBaseUrl: getBaseUrl,
  getHeaders: getHeaders,
  getParams: getParams,
  getFormData: getFormData,
  getData: getData,
  getAuth: getAuth,
  getConfig: getConfig,
  sendRequest: sendRequest
};